==> sunny-foundation-0.2.15.tgz: package/AccessControl/RBAC/PermissionsManager.ts <==
import { loadFromStorage, saveToSession } from 'sunny-js/util/storage'
import { isPlainObj } from 'sunny-js/util/object'
import {
  createImplement,
  invokeInterface,
  shouldImplement
} from 'sunny-js/util/module'

export interface IRBACPrivileges {
  [privilege: string]: any
}

export interface IRBACResource {
  privileges?: IRBACPrivileges
  [key: string]: any
}

export interface IRBACResources {
  [resource: string]: IRBACResource
}

// 定义模块接口
interface ModuleInterfaces {
  ajaxResources: () => Promise<any[]>
  // 用来创建resource key所用的属性名
  resourceProp?: string,
  ajaxPrivileges: (resource: IRBACResource) => Promise<any[]>
  // 用来创建Privilege key所用的属性名
  privilegeProp?: string,
}
const iRBAC = {
  resourceProp: 'path',
  privilegeProp: 'title'
} as ModuleInterfaces
export const implement = createImplement(iRBAC)

const ajaxResources = invokeInterface(
  () => iRBAC.ajaxResources,
  shouldImplement('iRBAC.ajaxResources')
)
const ajaxPrivileges = invokeInterface(
  () => iRBAC.ajaxPrivileges,
  shouldImplement('iRBAC.ajaxPrivileges')
)

let _permissions: IRBACResources

function getPermissions(): IRBACResources {
  return _permissions = _permissions || loadFromStorage('RBACPermissions') || {}
}

function RBACResource (resource: string, data?: object): IRBACResource {
  const cachedPermissions = getPermissions()
  if (isPlainObj(data)) {
    return cachedPermissions[resource] = data
  }
  return cachedPermissions[resource]
}

function RBACPrivileges (resource: string, privileges?: IRBACPrivileges): IRBACPrivileges {
  const cachedPermissions = getPermissions()
  if (isPlainObj(privileges)) {
    privileges = Object.assign(RBACPrivileges(resource) || {}, privileges)
    return cachedPermissions[resource].privileges = privileges
  }
  return cachedPermissions[resource]?.privileges
}

function saveRBACPermissions () {
  return saveToSession({
    RBACPermissions: getPermissions()
  }, true)
}

// RBAC权限控制模块所需的resources资源信息，后面的查询都会用到
export function getResources () {
  const resources: IRBACResources = _permissions || loadFromStorage('RBACPermissions')
  return resources
    ? Promise.resolve(resources)
    : ajaxResources().then(data => {
    data.forEach(item => RBACResource(item[iRBAC.resourceProp], item))
    saveRBACPermissions()
    return getPermissions()
  })
}

/**
 * 获取当前资源所拥有的Privileges信息
 * @param resource 默认为当前location.pathname
 */
export function getPrivileges (resource = location.pathname) {
  const privileges = RBACPrivileges(resource)
  if (privileges) {
    return Promise.resolve(privileges)
  }

  const objResource = RBACResource(resource)
  if (isPlainObj(objResource)) { // 有这个资源，就尝试查询该资源有可用privileges
    return ajaxPrivileges(objResource)
      .then(data => {
        let _privileges = {}
        data.forEach(item => _privileges[item[iRBAC.privilegeProp]] = item)
        _privileges = RBACPrivileges(resource, _privileges)
        Object.keys(_privileges).length > 0 && saveRBACPermissions()
        return _privileges
      })
      .catch(err => {
        console.error(err)
        return Promise.reject(err)
      })
  }

  // 没有resource默认返回一个空对象
  return Promise.resolve({})
}

// 测试某个Privilege是否存在与resource中
export function hasPrivilege (key: string, resource: string = location.pathname) {
  const privileges = RBACPrivileges(resource)
  if (!privileges) {
    console.warn('You should invoke ', getPrivileges, ' to prepare the data ahead for privilege testing')
  }
  return key in (privileges || {})
}

==> sunny-foundation-0.2.16.tgz: package/AccessControl/RBAC/PermissionsManager.ts <==
import { isPlainObj } from 'sunny-js/util/object'
import {
  createImplement,
  invokeInterface,
  shouldImplement
} from 'sunny-js/util/module'
import { isEmpty } from 'sunny-js/util/index'

export interface IRBACPrivileges {
  [privilege: string]: any
}

export interface IRBACResource {
  privileges?: IRBACPrivileges
  [key: string]: any
}

export interface IRBACResources {
  [resource: string]: IRBACResource
}

// 定义模块接口
interface ModuleInterfaces {
  ajaxResources: () => Promise<any[]>
  // 用来创建resource key所用的属性名
  resourceProp?: string,
  ajaxPrivileges: (resource: IRBACResource) => Promise<any[]>
  // 用来创建Privilege key所用的属性名
  privilegeProp?: string,
}
const iRBAC = {
  resourceProp: 'path',
  privilegeProp: 'title'
} as ModuleInterfaces
// noinspection JSUnusedGlobalSymbols
export const implement = createImplement(iRBAC)

const ajaxResources = invokeInterface(
  () => iRBAC.ajaxResources,
  shouldImplement('iRBAC.ajaxResources')
)
const ajaxPrivileges = invokeInterface(
  () => iRBAC.ajaxPrivileges,
  shouldImplement('iRBAC.ajaxPrivileges')
)

let _resources: IRBACResources

function cacheResources(): IRBACResources {
  return _resources = _resources || {}
}
// noinspection JSUnusedGlobalSymbols
export function clearCache () {
  _resources = null
}

function RBACResource (resource: string, data?: object): IRBACResource {
  const cachedPermissions = cacheResources()
  if (isPlainObj(data)) {
    return cachedPermissions[resource] = data
  }
  return cachedPermissions[resource]
}

function RBACPrivileges (resource: string, privileges?: IRBACPrivileges): IRBACPrivileges {
  const cachedPermissions = cacheResources()
  if (isPlainObj(privileges)) {
    privileges = Object.assign(RBACPrivileges(resource) || {}, privileges)
    return cachedPermissions[resource].privileges = privileges
  }
  return cachedPermissions[resource]?.privileges
}

// RBAC权限控制模块所需的resources资源信息，后面的查询都会用到
// noinspection JSUnusedGlobalSymbols
export function getResources () {
  const resources: IRBACResources = cacheResources()
  return !isEmpty(resources)
    ? Promise.resolve(resources)
    : ajaxResources().then(data => {
    data.forEach(item => RBACResource(item[iRBAC.resourceProp], item))
    return cacheResources()
  })
}

/**
 * 获取当前资源所拥有的Privileges信息
 * @param resource 默认为当前location.pathname
 */
export function getPrivileges (resource = location.pathname) {
  const privileges = RBACPrivileges(resource)
  if (privileges) {
    return Promise.resolve(privileges)
  }

  const objResource = RBACResource(resource)
  if (isPlainObj(objResource)) { // 有这个资源，就尝试查询该资源有可用privileges
    return ajaxPrivileges(objResource)
      .then(data => {
        const _privileges = {}
        data.forEach(item => _privileges[item[iRBAC.privilegeProp]] = item)
        return RBACPrivileges(resource, _privileges)
      })
      .catch(err => {
        console.error(err)
        return Promise.reject(err)
      })
  }

  // 没有resource默认返回一个空对象
  return Promise.resolve({})
}

// 测试某个Privilege是否存在与resource中
export function hasPrivilege (key: string, resource: string = location.pathname) {
  const privileges = RBACPrivileges(resource)
  if (!privileges) {
    console.warn('You should invoke ', getPrivileges, ' to prepare the data ahead for privilege testing')
  }
  return key in (privileges || {})
}
